#!/usr/bin/perl
# $Id: di624stats.pl,v 1.2 2004/10/06 23:39:59 ben Exp $
#
# di624stats.pl: Grab statistics from a D-Link DI-624 router.
# Copyright (C) 2004  Ben Ransford <ben@ransford.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
# USA.
#
######################################
#
# [ di624stats.pl ]
#
# fetch (screen-scrape) statistics from a D-Link DI-624 router, which
# doesn't seem to support SNMP.  optionally update an rrdtool database
# after retrieving statistics from the router.
#
# if you're using this script on a machine that's not behind your
# router, you'll have to enable remote administration from the IP
# address of the external machine.
#
# *** NOTE: the author of this script does NOT work for D-Link.
#
# if you're using the RRD update feature, create your RRD file using a
# command like this (see the rrdtool(1) man page for more information
# and explanation):
#
#	rrdtool create mystats.rrd \
#		--step=300 \
#		DS:in:COUNTER:600:0:12500000 \
#		DS:out:COUNTER:600:0:12500000 \
#		RRA:AVERAGE:0.5:1:288 \
#		RRA:AVERAGE:0.5:4:504 \
#		RRA:AVERAGE:0.5:144:730 \
#		RRA:MIN:0.5:1:288 \
#		RRA:MIN:0.5:4:504 \
#		RRA:MIN:0.5:144:730 \
#		RRA:MAX:0.5:1:288 \
#		RRA:MAX:0.5:4:504 \
#		RRA:MAX:0.5:144:730
#
# ben ransford <http://ben.ransford.org>, october 2004

use strict;
use Getopt::Std;
use LWP::UserAgent;
use Data::Dumper;
# comment out the 'use RRDs' line if you don't want an RRD file updated
use RRDs;

# add your own categories inside here (must be in router's stats page)
my %categories = (
	'WAN' => [],
	#'LAN' => [],
	#'WIRELESS 11g' => [],
);

# get the filename of this script
my $progname = $0;
$progname =~ s#.*/##;

# process command-line options.
#  -h [hostname or IP address of router] (required)
#  -P [port to connect to on the router; default 80]
#  -u [username for HTTP authentication; default "admin"]
#  -p [password for HTTP authentication] (required)
#  -r [realm for HTTP authentication; default "DI-624"]
#  -f [/path/to/rrd/database to update]
#  -d  DEBUG mode; produce output on stderr
my %opts;
getopts('h:P:u:p:r:f:d', \%opts);

my $router = $opts{'h'};
my $user = $opts{'u'} || 'admin';
my $pass = $opts{'p'};
my $realm = $opts{'r'} || 'DI-624';
my $DEBUG = $opts{'d'};

usage() unless ($router && $pass);

my $port = ($opts{'P'} =~ /^(\d+)$/) ? $1 : 80;
warn "Invalid port specification; try a number" unless $port eq $opts{'P'};

my $rrdfile;
if ($opts{'f'}) {
	$rrdfile = $opts{'f'};
	warn "Warning: write to RRD database will fail" unless -w $rrdfile;
}

# ready, set...
go();

####################

sub debug ($) {
	return unless $DEBUG;
	print STDERR shift, "\n";
}

sub usage {
	print STDERR "usage: ${progname} -h router_address [-P port]\n",
		     "       [-u username] -p password [-r realm] [-d]\n",
		     "       [-f /path/to/rrd/database]\n";
	exit 1;
}

sub rrdwrite ($) {
	my $category = shift;

	my $stats = $categories{$category};
	return unless ref($stats) eq "ARRAY";

	debug "About to update RRD file";
	RRDs::update($rrdfile, "--template", "in:out",
			"N:$$stats[0]:$$stats[1]");
}

sub go {
	my $ua = LWP::UserAgent->new;
	$ua->agent($progname);

	my $url = "http://${router}:${port}/st_stats.html";
	debug "URL == ${url}";
	my $req = HTTP::Request->new(GET => $url);
	$ua->credentials($req->uri()->host_port(), $realm, $user, $pass);
	my $res = $ua->request($req);
	debug "Fetched URL.";

	if ($res->is_success) {
		my @lines = split(/\n/, $res->content);
		my $looking = 0; # are we on the lookout for numbers?
		my $cur_cat = [];
		foreach my $line (@lines) {
			$line =~ s/^\s*//;
			$line =~ s/<[^>]*>//g;
			$line =~ s/\s*$//;

			if ($looking) {
				if ($line =~ /^(\d+)$/) {
					debug "Found stat $1";
					push @$cur_cat, $1;
				}

				# we have all info about the current
				# category, so put our antennae down
				if (scalar @$cur_cat == 2) {
					$looking = 0;
				};
				next;
			}

			# see if we're looking at a line that introduces
			# a stats category
			foreach my $cat (keys %categories) {
				if ($line eq $cat) {
					debug "Found category ${cat}";
					$looking = 1;
					$cur_cat = $categories{$cat};
				}
			}
		}

		debug Dumper(\%categories);
		if ($rrdfile) {
			foreach my $cat (keys %categories) {
				if (ref($categories{$cat}) eq "ARRAY") {
					rrdwrite($cat);
				}
			}
		}
	} else {
		warn $res->status_line . "\n";
	}
}
